"use strict";
var fs = require('fs');
var Hoek = require('hoek');
var angular2_universal_preview_1 = require('angular2-universal-preview');
var Runtime = (function () {
    function Runtime(options) {
        this.options = options;
        this.prebootScript = "\n    <preboot>\n      <link rel=\"stylesheet\" type=\"text/css\" href=\"/preboot/preboot.css\">\n      <script src=\"/preboot/preboot.js\"></script>\n      <script>preboot.start()</script>\n    </preboot>\n  ";
        this.angularScript = "\n    <!-- Browser polyfills -->\n    <script src=\"/node_modules/es6-shim/es6-shim.min.js\"></script>\n    <script src=\"/node_modules/systemjs/dist/system-polyfills.js\"></script>\n    <script src=\"/node_modules/angular2/bundles/angular2-polyfills.min.js\"></script>\n    <!-- SystemJS -->\n    <script src=\"/node_modules/systemjs/dist/system.js\"></script>\n    <!-- Angular2: Bundle -->\n    <script src=\"/node_modules/rxjs/bundles/Rx.js\"></script>\n    <script src=\"/node_modules/angular2/bundles/angular2.dev.js\"></script>\n    <script src=\"/node_modules/angular2/bundles/router.dev.js\"></script>\n    <script src=\"/node_modules/angular2/bundles/http.dev.js\"></script>\n    <script type=\"text/javascript\">\n      System.config({\n        \"baseURL\": \"/\",\n        \"defaultJSExtensions\": true\n      });\n    </script>\n  ";
        this.bootstrapButton = "\n    <div id=\"bootstrapButton\">\n      <style>\n       #bootstrapButton {\n        z-index:999999999;\n        position: absolute;\n        background-color: rgb(255, 255, 255);\n        padding: 0.5em;\n        border-radius: 3px;\n        border: 1px solid rgb(207, 206, 206);\n       }\n      </style>\n      <button onclick=\"bootstrap()\">\n        Bootstrap Angular2 Client\n      </button>\n    </div>\n  ";
        this.bootstrapApp = "\n    <script>\n      setTimeout(function() {\n        bootstrap();\n      });\n    </script>\n  ";
        this.renderPromise = angular2_universal_preview_1.renderToString;
    }
    Runtime.prototype.render = function (template, context, done) {
        var _this = this;
        context = Hoek.applyToDefaults(context, this.options);
        // bootstrap and render component to string
        var bootloader = context.bootloader;
        if (!context.bootloader) {
            context.bootloader = {
                document: angular2_universal_preview_1.parseDocument(template),
                providers: context.providers,
                componentProviders: context.componentProviders,
                platformProviders: context.platformProviders,
                directives: context.directives,
                preboot: context.preboot
            };
        }
        bootloader = angular2_universal_preview_1.Bootloader.create(context.bootloader);
        bootloader.serializeApplication()
            .then(function (html) { return done(null, _this.buildClientScripts(html, context)); })
            .catch(function (e) {
            console.error(e.stack);
            // if server fail then return client html
            done(null, _this.buildClientScripts(template, context));
        });
    };
    Runtime.prototype.bootstrapFunction = function (appUrl) {
        return "\n    <script>\n      function bootstrap() {\n        if (this.bootstraped) return;\n        this.bootstraped = true;\n        System.import(\"" + appUrl + "\")\n          .then(function(module) {\n            return module.main();\n          })\n          .then(function() {\n            preboot.complete();\n            var $bootstrapButton = document.getElementById(\"bootstrapButton\");\n            if ($bootstrapButton) { $bootstrapButton.remove(); }\n          });\n      }\n    </script>\n  ";
    };
    // TODO: find better ways to configure the App initial state
    // to pay off this technical debt
    // currently checking for explicit values
    Runtime.prototype.buildClientScripts = function (html, options) {
        if (!options || !options.buildClientScripts) {
            return html;
        }
        return html
            .replace(angular2_universal_preview_1.selectorRegExpFactory('preboot'), ((options.preboot === false) ? '' : this.prebootScript))
            .replace(angular2_universal_preview_1.selectorRegExpFactory('angular'), ((options.angular === false) ? '' : this.angularScript))
            .replace(angular2_universal_preview_1.selectorRegExpFactory('bootstrap'), ((options.bootstrap === false) ? (this.bootstrapButton +
            this.bootstrapFunction(options.componentUrl)) : (((options.client === undefined || options.server === undefined) ?
            '' : (options.client === false) ? '' : this.bootstrapButton) +
            this.bootstrapFunction(options.componentUrl) +
            ((options.client === false) ? '' : this.bootstrapApp))));
    };
    return Runtime;
}());
var hapiEngine = (function () {
    function hapiEngine() {
        this.helpers = {};
        this.partials = {};
    }
    hapiEngine.prototype.registerHelper = function (name, helper) {
        this.helpers[name] = helper;
    };
    hapiEngine.prototype.registerPartial = function (name, partial) {
        this.partials[name] = partial;
    };
    hapiEngine.prototype.compile = function (template, options, next) {
        var runtime = new Runtime(options);
        return next(null, function (context, options, callback) {
            return runtime.render(template, context, function (_, html) {
                return callback(_, html);
            });
        });
    };
    return hapiEngine;
}());
exports.hapiEngine = hapiEngine;
//# sourceMappingURL=engine.js.map